/* main.c - Application main entry point */

/*
 * Copyright (c) 2015-2016 Intel Corporation
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <sys/errno.h>
#include <zephyr/types.h>
#include <stddef.h>
#include <string.h>
#include <errno.h>
#include <zephyr/sys/printk.h>
#include <zephyr/sys/byteorder.h>
#include <zephyr/kernel.h>
//#include <zephyr/sys/sys_heap.h>

#include <zephyr/settings/settings.h>

#include <zephyr/bluetooth/bluetooth.h>
#include <zephyr/bluetooth/hci.h>
#include <zephyr/bluetooth/conn.h>
#include <zephyr/bluetooth/uuid.h>
#include <zephyr/bluetooth/gatt.h>

#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/drivers/sensor.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/debug/thread_analyzer.h>

#include <math.h>

#include <zephyr/logging/log.h>
LOG_MODULE_REGISTER(main);

#include "battery.h"
#include "sht4x.h"
#include "bmp180.h"
#include "sds011.h"
#include "bh1750.h"
#include "hal/nrf_gpiote.h"

static K_FIFO_DEFINE(fifo_adv_data);

static K_SEM_DEFINE(ble_init_ok, 0, 1);
static K_SEM_DEFINE(adv_send_ok, 0, 1);

#define BTHOME_MAXELT 10
enum captor_type { BATTERY, BATTERYVDD, TEMPERATURE, PRESSURE, ILLUMINANCE, VOLTAGE, VOLTAGEVDD, PM25, PM10, HUMIDITY };
uint8_t eltid[BTHOME_MAXELT] = { 0x01, 0x01, 0x02, 0x04, 0x05, 0x0c, 0x0c, 0x0d, 0x0e, 0x2e };
uint8_t eltsize[BTHOME_MAXELT] = { 1, 1, 2, 3, 3, 2, 2, 2, 2, 1};
uint8_t eltenabled[BTHOME_MAXELT] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
uint8_t eltenabled2[BTHOME_MAXELT] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

struct bthome_adv_t {
    void *fifo_reserved;
    uint8_t type;
    int16_t data_s16;
    uint16_t data_u16;
    uint8_t data_u8;
    uint32_t data_u32;
};

uint8_t mfg_bthome_tosend = 1;

   static uint8_t mfg_bthome_data[28];
   uint8_t mfg_bthome_sizetosend = 0;
   uint8_t mfg_bthome_numelt = 0;
   uint8_t bthome_datapos[BTHOME_MAXELT];

   static uint8_t mfg_bthome_data2[28];
   uint8_t mfg_bthome_sizetosend2 = 0;
   uint8_t mfg_bthome_numelt2 = 0;
   uint8_t bthome_datapos2[BTHOME_MAXELT];

 

    struct captor_wait {
        enum captor_type captor;
        uint16_t wait;
    };

    struct captor_wait params[] = { {BATTERY, 120 }, {TEMPERATURE, 60}, {PM25, 60} };


    /*static const struct bt_data ad[] = {
        BT_DATA_BYTES(BT_DATA_FLAGS, (BT_LE_AD_GENERAL | BT_LE_AD_NO_BREDR)),
        BT_DATA(BT_DATA_SVC_DATA16, mfg_bthome_data, 23),
    };*/

#define GPIO_PORT DT_NODELABEL(gpio0)

#define STACKSIZE 2048
#define PRIORITY 7

K_THREAD_STACK_DEFINE(stack_wq_pms, STACKSIZE);
K_THREAD_STACK_DEFINE(stack_wq_i2c, STACKSIZE);

struct k_work_q workqueue_pms;
struct k_work_q workqueue_i2c;

    static const struct battery_level_point battery_levels[] = {
        /* "Curve" here eyeballed from captured data for the [Adafruit
         * 3.7v 2000 mAh](https://www.adafruit.com/product/2011) LIPO
         * under full load that started with a charge of 3.96 V and
         * dropped about linearly to 3.58 V over 15 hours.  It then
         * dropped rapidly to 3.10 V over one hour, at which point it
         * stopped transmitting.
         *
         * Based on eyeball comparisons we'll say that 15/16 of life
         * goes between 3.95 and 3.55 V, and 1/16 goes between 3.55 V
         * and 3.1 V.
         */

        { 10000, 3950 },
        { 625, 3550 },
        { 0, 3100 },
    };
    static const struct battery_level_point battery_nimh_levels[] = {
        /* "Curve" here eyeballed from captured data for the [Adafruit
         * 3.7v 2000 mAh](https://www.adafruit.com/product/2011) LIPO
         * under full load that started with a charge of 3.96 V and
         * dropped about linearly to 3.58 V over 15 hours.  It then
         * dropped rapidly to 3.10 V over one hour, at which point it
         * stopped transmitting.
         *
         * Based on eyeball comparisons we'll say that 15/16 of life
         * goes between 3.95 and 3.55 V, and 1/16 goes between 3.55 V
         * and 3.1 V.
         */

        { 10000, 2800 },
        { 9000, 2600 },
        { 5000, 2440 },
        { 0, 2300 },
    };


    
    //#define UART_DEVICE_NODE DT_CHOSEN(zephyr_shell_uart)


    /* static void connected(struct bt_conn *conn, uint8_t err)
       {
       if (err) {
       printk("Connection failed (err 0x%02x)\n", err);
       } else {
       printk("Connected\n");
       }
       }

       static void disconnected(struct bt_conn *conn, uint8_t reason)
       {
       printk("Disconnected (reason 0x%02x)\n", reason);
       }*/

  /*  uint32_t getTs()
    {e
        uint32_t ts = (k_uptime_get_32()/1000);
        // LOG_INF("getTs: %d", ts);
        return ts;
    }

   uint16_t findsmallestwait()
    {
        uint16_t minval = UINT16_MAX;
        uint32_t ts = getTs();
        for(int i = 0; i < sizeof(params) / sizeof(params[0]); i++)
        {
            if(params[i].wait != 0) {
                //LOG_INF("%s %d", params[i].captor[j].captor, params[i].captor[j].wait);
                uint32_t wait = (uint32_t)params[i].wait;
                if((uint16_t)(wait-(ts % wait)) < minval) { minval = (uint16_t)(wait-(ts % wait)); }
            }
        }
        return minval;
    }

    uint16_t findiftoprobe(enum captor_type captorp, bool record)
    {
        uint32_t ts = getTs();
        //LOG_INF("findiftoprobe: %s", captorp);
        for(int i = 0; i < sizeof(params) / sizeof(params[0]); i++)
        {
            if(captorp == params[i].captor) {
                return (uint16_t)(ts % (uint32_t)params[i].wait);
            }
        }
        return UINT16_MAX;

    }
*/

    static void bt_ready(void)
    {
        int err;

        printk("Bluetooth initialized\n");

        /*if (IS_ENABLED(CONFIG_SETTINGS)) {
          settings_load();
          }*/


     struct bt_data ad[] = {
        BT_DATA_BYTES(BT_DATA_FLAGS, (BT_LE_AD_GENERAL | BT_LE_AD_NO_BREDR)),
        BT_DATA(BT_DATA_SVC_DATA16, mfg_bthome_data, mfg_bthome_sizetosend),
    };
//        err = bt_le_adv_start(BT_LE_ADV_PARAM(BT_LE_ADV_OPT_USE_IDENTITY | BT_LE_ADV_OPT_USE_NAME, BT_GAP_ADV_SLOW_INT_MIN,BT_GAP_ADV_SLOW_INT_MAX, NULL), ad, ARRAY_SIZE(ad), NULL, 0);
        err = bt_le_adv_start(BT_LE_ADV_PARAM(BT_LE_ADV_OPT_USE_IDENTITY, BT_GAP_ADV_SLOW_INT_MIN,BT_GAP_ADV_SLOW_INT_MAX, NULL), ad, ARRAY_SIZE(ad), NULL, 0);
        if (err) {
            printk("Advertising failed to start (err %d)\n", err);
            return;
        }

        printk("Advertising successfully started\n");
    }

    /* static void auth_passkey_display(struct bt_conn *conn, unsigned int passkey)
       {
       char addr[BT_ADDR_LE_STR_LEN];

       bt_addr_le_to_str(bt_conn_get_dst(conn), addr, sizeof(addr));

       printk("Passkey for %s: %06u\n", addr, passkey);
       }

       static void auth_cancel(struct bt_conn *conn)
       {
       char addr[BT_ADDR_LE_STR_LEN];

       bt_addr_le_to_str(bt_conn_get_dst(conn), addr, sizeof(addr));

       printk("Pairing cancelled: %s\n", addr);
       }

       static struct bt_conn_auth_cb auth_cb = {
       .passkey_display = auth_passkey_display,
       .passkey_entry = NULL,
       .cancel = auth_cancel,
       };*/

    /*bool test_bthome_data_set()
    {
        for(int i = 0; i < sizeof(bthome_datapos); i++)
        {
            if(mfg_bthome_data[bthome_datapos[i]] == 0x00)
            {
                //printk("BTHome POS %d empty\n", bthome_datapos[i]);
                return false;
            }
        }
        return true;
    }*/

/*void shift_bthome_datapos(uint8_t value)
{
  for(int i = 0; i < BTHOME_MAXELT; i++)
  {
    if(bthome_datapos[i] == 255)
    {
      bthome_datapos[i] = bthome_datapos[i] + value;
    }
  }

}
uint8_t mfg_bthome_addelt(uint8_t eltid, uint8_t eltsize)
{
  if(mfg_bthome_numelt == 0)
  {
    mfg_bthome_numelt++;
    mfg_bthome_data[3] = eltid;
    mfg_bthome_sizetosend = mfg_bthome_sizetosend + eltsize +1;
    return 3;
  } else {
    if(eltid < mfg_bthome_data[3])
    {
      mfg_bthome_sizetosend = mfg_bthome_sizetosend + eltsize +1;
      for(int i = 0; i < eltsize+1; i++)
      {
        for(int j = mfg_bthome_sizetosend; j > 2; j--)
        {
          mfg_bthome_data[j] = mfg_bthome_data[j-1];
        }
    }
      shift_bthome_datapos(eltsize+1);
      mfg_bthome_data[3] = eltid;
      return 3;
    } else {
      mfg_bthome_data[mfg_bthome_sizetosend] = eltid;
      mfg_bthome_sizetosend = mfg_bthome_sizetosend + eltsize +1;
      return mfg_bthome_sizetosend - eltsize - 1;
    }
  }
}
*/

void mfg_bthome_build()
{
  for(int i = 0; i < BTHOME_MAXELT; i++)
  {
    if(eltenabled[i] == 1)
    {
    mfg_bthome_numelt++;
      mfg_bthome_data[mfg_bthome_sizetosend] = eltid[i];
      bthome_datapos[i] = mfg_bthome_sizetosend;
      mfg_bthome_sizetosend = mfg_bthome_sizetosend + eltsize[i] + 1;
    }
  }
}

void mfg_bthome_build2()
{
  for(int i = 0; i < BTHOME_MAXELT; i++)
  {
    if(eltenabled2[i] == 1)
    {
    mfg_bthome_numelt2++;
      mfg_bthome_data2[mfg_bthome_sizetosend2] = eltid[i];
      bthome_datapos2[i] = mfg_bthome_sizetosend2;
      mfg_bthome_sizetosend2 = mfg_bthome_sizetosend2 + eltsize[i] + 1;
    }
  }
}
    static void work_publish_adv_handler(struct k_work *work)
    {
     struct bt_data ad[] = {
        BT_DATA_BYTES(BT_DATA_FLAGS, (BT_LE_AD_GENERAL | BT_LE_AD_NO_BREDR)),
        BT_DATA(BT_DATA_SVC_DATA16, mfg_bthome_data, mfg_bthome_sizetosend),
    };
     struct bt_data ad2[] = {
        BT_DATA_BYTES(BT_DATA_FLAGS, (BT_LE_AD_GENERAL | BT_LE_AD_NO_BREDR)),
        BT_DATA(BT_DATA_SVC_DATA16, mfg_bthome_data2, mfg_bthome_sizetosend2),
    };
  if(mfg_bthome_tosend == 1) {
                    printk("Update ADV2\n");
                    bt_le_adv_update_data(ad2, ARRAY_SIZE(ad2), NULL, 0);
    mfg_bthome_tosend = 2;
  } else {
                    printk("Update ADV1\n");
                    bt_le_adv_update_data(ad, ARRAY_SIZE(ad), NULL, 0);
    mfg_bthome_tosend = 1;

  }

}
    K_WORK_DEFINE(work_publish_adv, work_publish_adv_handler);

void timer_publish_adv_handler(struct k_timer *stm)
{
    k_work_submit(&work_publish_adv);
                }
    K_TIMER_DEFINE(timer_publish_adv, timer_publish_adv_handler, NULL);


    void publish_adv_thread()
    {
        bool adv_activate = false;
        uint8_t numeltreceived = 0;
uint8_t eltreceived[BTHOME_MAXELT] = { 0, 0, 0, 0, 0, 0, 0 };
   //static uint8_t mfg_bthome_data[] = { 0xd2, 0xfc, 0x40, 0x01, 0x00, 0x0c, 0x00, 0x00, 0x02, 0x00, 0x00, 0x2e, 0x00, 0x04, 0x00, 0x00, 0x00, 0x0d, 0x00, 0x00, 0x0e, 0x00, 0x00 };
    //uint8_t bthome_datapos[] = { 4, 6, 9, 12, 14, 18, 21 };

        k_sem_take(&ble_init_ok, K_FOREVER);

        for (;;) {
    k_sem_take(&adv_send_ok, K_FOREVER);
            while(k_fifo_is_empty(&fifo_adv_data) == 0)
    {
            struct bthome_adv_t *request = k_fifo_get(&fifo_adv_data, K_NO_WAIT);
/*            if(bthome_datapos[request->type] == 255)
      {
            bthome_datapos[request->type] = mfg_bthome_addelt(eltid[request->type], eltsize[request->type]);        
      }*/
            
      if(eltreceived[request->type] == 0) {
      numeltreceived++;
      eltreceived[request->type] = 1;
      }
      //LOG_INF("Fill BT for type %d", request->type);
            if(request->type == VOLTAGE)
            {
                mfg_bthome_data2[bthome_datapos2[VOLTAGE]+2] = request->data_u16 >> 8;
                mfg_bthome_data2[bthome_datapos2[VOLTAGE]+1] = request->data_u16;
            } else if(request->type == BATTERY)
            {
                mfg_bthome_data2[bthome_datapos2[BATTERY]+1] = request->data_u8;
            } else if(request->type == VOLTAGEVDD)
            {
                mfg_bthome_data2[bthome_datapos2[VOLTAGEVDD]+2] = request->data_u16 >> 8;
                mfg_bthome_data2[bthome_datapos2[VOLTAGEVDD]+1] = request->data_u16;
            } else if(request->type == BATTERYVDD)
            {
                mfg_bthome_data2[bthome_datapos2[BATTERYVDD]+1] = request->data_u8;
            } else if(request->type == TEMPERATURE)
            {
                mfg_bthome_data[bthome_datapos[TEMPERATURE]+2] = request->data_s16 >> 8;
                mfg_bthome_data[bthome_datapos[TEMPERATURE]+1] = request->data_s16;
            } else if(request->type == HUMIDITY)
            {
                mfg_bthome_data[bthome_datapos[HUMIDITY]+1] = request->data_u8;
            } else if(request->type == PRESSURE)
            {
                mfg_bthome_data[bthome_datapos[PRESSURE]+3] = request->data_u32 >> 16;
                mfg_bthome_data[bthome_datapos[PRESSURE]+2] = request->data_u32 >> 8;
                mfg_bthome_data[bthome_datapos[PRESSURE]+1] = request->data_u32;
            } else if(request->type == ILLUMINANCE)
            {
                mfg_bthome_data[bthome_datapos[ILLUMINANCE]+3] = request->data_u32 >> 16;
                mfg_bthome_data[bthome_datapos[ILLUMINANCE]+2] = request->data_u32 >> 8;
                mfg_bthome_data[bthome_datapos[ILLUMINANCE]+1] = request->data_u32;
            } else if(request->type == PM25)
            {
                mfg_bthome_data[bthome_datapos[PM25]+2] = request->data_u16 >> 8;
                mfg_bthome_data[bthome_datapos[PM25]+1] = request->data_u16;
            } else if(request->type == PM10)
            {
                mfg_bthome_data[bthome_datapos[PM10]+2] = request->data_u16 >> 8;
                mfg_bthome_data[bthome_datapos[PM10]+1] = request->data_u16;
            }

            k_free(request);
    }
    if(numeltreceived == (mfg_bthome_numelt + mfg_bthome_numelt2)) {
                if(adv_activate == false)
                {
                    bt_ready();
                    adv_activate = true;
        k_timer_start(&timer_publish_adv, K_SECONDS(150), K_SECONDS(150));
                } 
    }
    /*
    if(numeltreceived == mfg_bthome_numelt) {
                if(adv_activate == false)
                {
                    bt_ready();
                    adv_activate = true;
                } else {
                    printk("Update ADV\n");
     struct bt_data ad[] = {
        BT_DATA(BT_DATA_SVC_DATA16, mfg_bthome_data, mfg_bthome_sizetosend),
    };
                    bt_le_adv_update_data(ad, ARRAY_SIZE(ad), NULL, 0);
                }
            }
    */
  }
    }
    K_THREAD_DEFINE(publish_adv_thread_id, STACKSIZE, publish_adv_thread, NULL, NULL, NULL, PRIORITY, 0, 0);

/*void print_heap_info(void) 
{
extern struct k_heap _system_heap;
struct sys_memory_stats stats;
    sys_heap_runtime_stats_get(&_system_heap.heap, &stats);

    printk("\n");
    printk("INFO: Allocated Heap = %zu\n", stats.allocated_bytes);
    printk("INFO: Free Heap = %zu\n", stats.free_bytes);
    printk("INFO: Max Allocated Heap = %zu\n", stats.max_allocated_bytes);
    printk("\n");

    return;
}*/

    static void work_battery_handler(struct k_work *work)
    {
        struct bthome_adv_t *bthome_adv_voltage = k_malloc(sizeof(*bthome_adv_voltage));
        bthome_adv_voltage->type = VOLTAGE;
        battery_setup();
        bthome_adv_voltage->data_u16 = (uint16_t) battery_sample();
        k_fifo_put(&fifo_adv_data, bthome_adv_voltage);

        struct bthome_adv_t *bthome_adv_battery = k_malloc(sizeof(*bthome_adv_battery));
        bthome_adv_battery->type = BATTERY;
        bthome_adv_battery->data_u8 = round(battery_level_pptt(bthome_adv_voltage->data_u16, battery_levels)/100);
        k_fifo_put(&fifo_adv_data, bthome_adv_battery);
        printk("Battery: %d mV (%d%%)\n",bthome_adv_voltage->data_u16, bthome_adv_battery->data_u8);

        struct bthome_adv_t *bthome_adv_voltagevdd = k_malloc(sizeof(*bthome_adv_voltagevdd));
        bthome_adv_voltagevdd->type = VOLTAGEVDD;
        batteryvdd_setup();
        bthome_adv_voltagevdd->data_u16 = (uint16_t) batteryvdd_sample();
        k_fifo_put(&fifo_adv_data, bthome_adv_voltagevdd);

        struct bthome_adv_t *bthome_adv_batteryvdd = k_malloc(sizeof(*bthome_adv_batteryvdd));
        bthome_adv_batteryvdd->type = BATTERYVDD;
        bthome_adv_batteryvdd->data_u8 = round(battery_level_pptt(bthome_adv_voltagevdd->data_u16, battery_nimh_levels)/100);
        k_fifo_put(&fifo_adv_data, bthome_adv_batteryvdd);
        printk("Battery VDD: %d mV (%d%%)\n",bthome_adv_voltagevdd->data_u16, bthome_adv_batteryvdd->data_u8);
        k_sem_give(&adv_send_ok);

    }
    K_WORK_DEFINE(work_battery, work_battery_handler);
void timer_battery_handler(struct k_timer *stm)
{
  k_work_submit(&work_battery);
}
    K_TIMER_DEFINE(timer_battery, timer_battery_handler, NULL);

    static void work_temperature_handler(struct k_work *work)
    {
        struct bthome_adv_t *bthome_adv_temperature = k_malloc(sizeof(*bthome_adv_temperature));
        sht4x_probe();
        bthome_adv_temperature->type = TEMPERATURE;
        bthome_adv_temperature->data_s16 = (int16_t)(sht4x_ReadTempeature()*100);
        k_fifo_put(&fifo_adv_data, bthome_adv_temperature);
        printk("Temperature: %d°\n",bthome_adv_temperature->data_s16);


        struct bthome_adv_t *bthome_adv_humidity = k_malloc(sizeof(*bthome_adv_humidity));
        bthome_adv_humidity->type = HUMIDITY;
        bthome_adv_humidity->data_u8 = (uint8_t)(sht4x_ReadHumidity());
        k_fifo_put(&fifo_adv_data, bthome_adv_humidity);
        k_sem_give(&adv_send_ok);
        printk("Humidity: %d pc \n",bthome_adv_humidity->data_u8);
    }
    K_WORK_DEFINE(work_temperature, work_temperature_handler);

void timer_temperature_handler(struct k_timer *stm)
{
    k_work_submit_to_queue(&workqueue_i2c, &work_temperature);
}
    K_TIMER_DEFINE(timer_temperature, timer_temperature_handler, NULL);

static void work_pressure_handler(struct k_work *work)
    {
        struct bthome_adv_t *bthome_adv_pressure = k_malloc(sizeof(*bthome_adv_pressure));
        bthome_adv_pressure->type = PRESSURE;
        bthome_adv_pressure->data_u32 = bmp180_readPressure();
        k_fifo_put(&fifo_adv_data, bthome_adv_pressure);
        k_sem_give(&adv_send_ok);
        printk("Pressure: %d Pa \n",bthome_adv_pressure->data_u32);
    }
    K_WORK_DEFINE(work_pressure, work_pressure_handler);
void timer_pressure_handler(struct k_timer *stm)
{
    k_work_submit_to_queue(&workqueue_i2c, &work_pressure);
}
    K_TIMER_DEFINE(timer_pressure, timer_pressure_handler, NULL);

static void work_illuminance_handler(struct k_work *work)
    {
        struct bthome_adv_t *bthome_adv_illuminance = k_malloc(sizeof(*bthome_adv_illuminance));
        bh1750_probe();
        bthome_adv_illuminance->type = ILLUMINANCE;
        bthome_adv_illuminance->data_u32 = (uint32_t) (bh1750_ReadLight()*100);
        k_fifo_put(&fifo_adv_data, bthome_adv_illuminance);
        k_sem_give(&adv_send_ok);
        printk("Illuminance: %d lux \n",bthome_adv_illuminance->data_u32);
    }
    K_WORK_DEFINE(work_illuminance, work_illuminance_handler);
void timer_illuminance_handler(struct k_timer *stm)
{
    k_work_submit_to_queue(&workqueue_i2c, &work_illuminance);
}
    K_TIMER_DEFINE(timer_illuminance, timer_illuminance_handler, NULL);

static void work_pms_handler(struct k_work *work)
    {
  sds011_probe();
    struct bthome_adv_t *bthome_adv_pm25 = k_malloc(sizeof(*bthome_adv_pm25));
            bthome_adv_pm25->type = PM25;
            bthome_adv_pm25->data_u16 = sds011_ReadPM25();
            k_fifo_put(&fifo_adv_data, bthome_adv_pm25);
            printk("PM25: %d\n",bthome_adv_pm25->data_u16);
        struct bthome_adv_t *bthome_adv_pm10 = k_malloc(sizeof(*bthome_adv_pm10));
            bthome_adv_pm10->type = PM10;
            bthome_adv_pm10->data_u16 = sds011_ReadPM10();
            k_fifo_put(&fifo_adv_data, bthome_adv_pm10);
            printk("PM10: %d\n",bthome_adv_pm10->data_u16);
        k_sem_give(&adv_send_ok);
}
    K_WORK_DEFINE(work_pms, work_pms_handler);


void timer_pms_handler(struct k_timer *stm)
{
    k_work_submit_to_queue(&workqueue_pms, &work_pms);
}
    K_TIMER_DEFINE(timer_pms, timer_pms_handler, NULL);
   
    int main(void)
    {
        LOG_WRN("Hello, It's TuxOutdooor");
        int err;

        //     bt_passkey_set(192847);
      mfg_bthome_data[0]=0xd2;
      mfg_bthome_data[1]=0xfc;
      mfg_bthome_data[2]=0x40;
      mfg_bthome_sizetosend=3;

      mfg_bthome_data2[0]=0xd2;
      mfg_bthome_data2[1]=0xfc;
      mfg_bthome_data2[2]=0x40;
      mfg_bthome_sizetosend2=3;


      for(int i = 0; i < BTHOME_MAXELT; i++)
  {
    bthome_datapos[i] = 255;
  }

        err = bt_enable(NULL);
        if (err) {
            printk("Bluetooth init failed (err %d)\n", err);
            return 0;
        }


        const struct device *dev = DEVICE_DT_GET(GPIO_PORT);

        gpio_pin_configure(dev, 20, GPIO_OUTPUT);
        gpio_pin_set(dev, 20, 0);
        //bt_conn_auth_cb_register(&auth_cb);
        //static const struct device *const uart_dev = DEVICE_DT_GET(UART_DEVICE_NODE);
  
  
k_work_queue_init(&workqueue_i2c);
k_work_queue_start(&workqueue_i2c, stack_wq_i2c, K_THREAD_STACK_SIZEOF(stack_wq_i2c), PRIORITY, NULL);
k_work_queue_init(&workqueue_pms);
k_work_queue_start(&workqueue_pms, stack_wq_pms, K_THREAD_STACK_SIZEOF(stack_wq_pms), PRIORITY, NULL);

  k_msleep(250);

  if(sht4x_setup() == 0) { 
    printk("SHT4x enabled\n");
    k_work_submit_to_queue(&workqueue_i2c, &work_temperature);
    k_timer_start(&timer_temperature, K_SECONDS(300), K_SECONDS(300));
  eltenabled[TEMPERATURE] = 1;
  eltenabled[HUMIDITY] = 1;
  } else {
    printk("SHT4x disabled\n");
  }

  k_msleep(250);


  if(bmp180_setup() == 0) { 
    printk("BMP180 enabled\n");
    k_work_submit_to_queue(&workqueue_i2c, &work_pressure);
    k_timer_start(&timer_pressure, K_SECONDS(1800), K_SECONDS(1800));
  eltenabled[PRESSURE] = 1;
  } else {
    printk("BMP180 disabled\n");
  }


  k_msleep(250);

  if(sds011_setup() == 0) {
    printk("SDS011 enabled\n");
    k_work_submit_to_queue(&workqueue_pms, &work_pms);
  eltenabled[PM10] = 1;
  eltenabled[PM25] = 1;
  k_timer_start(&timer_pms, K_SECONDS(1800), K_SECONDS(1800));
  } else {
    printk("SDS011 disabled\n");
  }

  k_msleep(2000);

   if(bh1750_setup() == 0) { 
    printk("BH1750 enabled\n");
    k_work_submit_to_queue(&workqueue_i2c, &work_illuminance);
    k_timer_start(&timer_illuminance, K_SECONDS(300), K_SECONDS(300));
    eltenabled[ILLUMINANCE] = 1;
  } else {
    printk("BH1750 disabled\n");
  }
 
  k_msleep(250);
 k_work_submit(&work_battery);
  k_timer_start(&timer_battery, K_SECONDS(3600), K_SECONDS(3600));
  eltenabled2[BATTERY] = 1;
  eltenabled2[VOLTAGE] = 1;
  eltenabled2[BATTERYVDD] = 1;
  eltenabled2[VOLTAGEVDD] = 1;

  mfg_bthome_build();
  mfg_bthome_build2();

        k_sem_give(&ble_init_ok);
/*        for(;;)
        {
            if(findiftoprobe(BATTERY, true) < 5)
            {
                k_work_submit(&work_battery);
            }
            if(findiftoprobe(TEMPERATURE, true) < 5)
            {
                k_work_submit(&work_temperature);
            }
            if(findiftoprobe(PM25, true) < 5)
            {
                k_work_submit(&work_pms);
            }
            k_msleep(1000*findsmallestwait());

        } */

        /*int len = 0;
          int pm10_serial = 0;
          int pm25_serial = 0;
          int checksum_is;
          int checksum_ok = 0;
          int error = 1;
          float p10 = 0.0;
          float p25 = 0.0;
          for(;;)	{
          char value;
          if(uart_poll_in(uart_dev, &value) == 0)
          {
          printk("UART char received: 0x%x\n", value);
          printk("len %d\n", len);
          switch (len) {
          case (0): if (value != 170) { len = -1; }; break;
          case (1): if (value != 192) { len = -1; }; break;
          case (2): pm25_serial = value; checksum_is = value; break;
          case (3): pm25_serial += (value << 8); checksum_is += value; break;
          case (4): pm10_serial = value; checksum_is += value; break;
          case (5): pm10_serial += (value << 8); checksum_is += value; break;
          case (6): checksum_is += value; break;
          case (7): checksum_is += value; break;
          case (8): if (value == (checksum_is % 256)) { checksum_ok = 1; } else { len = -1; }; break;
          case (9): if (value != 171) { len = -1; }; break;
          }
          len++;
          if (len == 10 && checksum_ok == 1) {
          p10 = (float)pm10_serial/10.0;
          p25 = (float)pm25_serial/10.0;
          len = 0; checksum_ok = 0; pm10_serial = 0.0; pm25_serial = 0.0; checksum_is = 0;
          error = 0;
          printk("p10: %f p25: %f\n", p10, p25);
          }
          } else {
        //printk("No UART Data\n");
        k_msleep(100);
        }
        }*/

        return 0;
    }
